@@ -65,6 +65,19 @@ static void soup_cache_session_feature_init (SoupSessionFeatureInterface *featur
  */
 #define SOUP_CACHE_CURRENT_VERSION 5
 
+#define OLD_SOUP_CACHE_FILE "soup.cache"
+#define SOUP_CACHE_FILE "soup.cache2"
+
+#define SOUP_CACHE_HEADERS_FORMAT "{ss}"
+#define SOUP_CACHE_PHEADERS_FORMAT "(sbuuuuuqa" SOUP_CACHE_HEADERS_FORMAT ")"
+#define SOUP_CACHE_ENTRIES_FORMAT "(qa" SOUP_CACHE_PHEADERS_FORMAT ")"
+
+/* Basically the same format than above except that some strings are
+   prepended with &. This way the GVariant returns a pointer to the
+   data instead of duplicating the string */
+#define SOUP_CACHE_DECODE_HEADERS_FORMAT "{&s&s}"
+
+
 typedef struct _SoupCacheEntry {
 	guint32 key;
 	char *uri;
@@ -73,13 +86,9 @@ typedef struct _SoupCacheEntry {
 	gsize length;
 	guint32 corrected_initial_age;
 	guint32 response_time;
-	SoupBuffer *current_writing_buffer;
 	gboolean dirty;
-	gboolean got_body;
 	gboolean being_validated;
 	SoupMessageHeaders *headers;
-	GOutputStream *stream;
-	GError *error;
 	guint32 hits;
 	GCancellable *cancellable;
 	guint16 status_code;
@@ -106,6 +115,10 @@ typedef struct {
 	gulong got_body_handler;
 	gulong restarted_handler;
 	GQueue *buffer_queue;
+	gboolean got_body;
+	SoupBuffer *current_writing_buffer;
+	GError *error;
+	GOutputStream *ostream;
 } SoupCacheWritingFixture;
 
 enum {
@@ -120,7 +133,7 @@ G_DEFINE_TYPE_WITH_CODE (SoupCache, soup_cache, G_TYPE_OBJECT,
 			 G_IMPLEMENT_INTERFACE (SOUP_TYPE_SESSION_FEATURE,
 						soup_cache_session_feature_init))
 
-static gboolean soup_cache_entry_remove (SoupCache *cache, SoupCacheEntry *entry);
+static gboolean soup_cache_entry_remove (SoupCache *cache, SoupCacheEntry *entry, gboolean purge);
 static void make_room_for_new_entry (SoupCache *cache, guint length_to_add);
 static gboolean cache_accepts_entries_of_size (SoupCache *cache, guint length_to_add);
 static gboolean write_next_buffer (SoupCacheEntry *entry, SoupCacheWritingFixture *fixture);
@@ -260,17 +273,10 @@ get_cacheability (SoupCache *cache, SoupMessage *msg)
  * and also unref's the GFile object representing it.
  */
 static void
-soup_cache_entry_free (SoupCacheEntry *entry, GFile *file)
+soup_cache_entry_free (SoupCacheEntry *entry)
 {
-	if (file) {
-		g_file_delete (file, NULL, NULL);
-		g_object_unref (file);
-	}
-
 	g_free (entry->uri);
-	g_clear_pointer (&entry->current_writing_buffer, soup_buffer_free);
 	g_clear_pointer (&entry->headers, soup_message_headers_free);
-	g_clear_error (&entry->error);
 	g_clear_object (&entry->cancellable);
 
 	g_slice_free (SoupCacheEntry, entry);
@@ -442,10 +448,7 @@ soup_cache_entry_new (SoupCache *cache, SoupMessage *msg, time_t request_time, t
 
 	entry = g_slice_new0 (SoupCacheEntry);
 	entry->dirty = FALSE;
-	entry->current_writing_buffer = NULL;
-	entry->got_body = FALSE;
 	entry->being_validated = FALSE;
-	entry->error = NULL;
 	entry->status_code = msg->status_code;
 	entry->response_time = response_time;
 	entry->uri = soup_uri_to_string (soup_message_get_uri (msg), FALSE);
@@ -502,6 +505,9 @@ soup_cache_writing_fixture_free (SoupCacheWritingFixture *fixture)
 		g_signal_handler_disconnect (fixture->msg, fixture->got_body_handler);
 	if (g_signal_handler_is_connected (fixture->msg, fixture->restarted_handler))
 		g_signal_handler_disconnect (fixture->msg, fixture->restarted_handler);
+	g_clear_pointer (&fixture->current_writing_buffer, soup_buffer_free);
+	g_clear_object (&fixture->ostream);
+	g_clear_error (&fixture->error);
 	g_queue_foreach (fixture->buffer_queue, (GFunc) soup_buffer_free, NULL);
 	g_queue_free (fixture->buffer_queue);
 	g_object_unref (fixture->msg);
@@ -523,7 +529,7 @@ close_ready_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *
 	GOutputStream *stream = G_OUTPUT_STREAM (source);
 	goffset content_length;
 
-	g_warn_if_fail (entry->error == NULL);
+	g_warn_if_fail (fixture->error == NULL);
 
 	/* FIXME: what do we do on error ? */
 
@@ -531,7 +537,7 @@ close_ready_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *
 		g_output_stream_close_finish (stream, result, NULL);
 		g_object_unref (stream);
 	}
-	entry->stream = NULL;
+	fixture->ostream = NULL;
 
 	content_length = soup_message_headers_get_content_length (entry->headers);
 
@@ -540,8 +546,7 @@ close_ready_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *
 	   too much for the cache */
 	if (g_cancellable_is_cancelled (entry->cancellable)) {
 		entry->dirty = FALSE;
-		soup_cache_entry_remove (cache, entry);
-		soup_cache_entry_free (entry, get_file_from_entry (cache, entry));
+		soup_cache_entry_remove (cache, entry, TRUE);
 		entry = NULL;
 	} else if ((soup_message_headers_get_encoding (entry->headers) == SOUP_ENCODING_CHUNKED) ||
 		   entry->length != (gsize) content_length) {
@@ -567,23 +572,17 @@ close_ready_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *
 			cache->priv->size += length_to_add;
 		} else {
 			entry->dirty = FALSE;
-			soup_cache_entry_remove (cache, entry);
-			soup_cache_entry_free (entry, get_file_from_entry (cache, entry));
+			soup_cache_entry_remove (cache, entry, TRUE);
 			entry = NULL;
 		}
 	}
 
 	if (entry) {
 		entry->dirty = FALSE;
-		entry->got_body = FALSE;
+		fixture->got_body = FALSE;
 
-		if (entry->current_writing_buffer) {
-			soup_buffer_free (entry->current_writing_buffer);
-			entry->current_writing_buffer = NULL;
-		}
-
-		g_object_unref (entry->cancellable);
-		entry->cancellable = NULL;
+		g_clear_pointer (&fixture->current_writing_buffer, soup_buffer_free);
+		g_clear_object (&entry->cancellable);
 	}
 
 	cache->priv->n_pending--;
@@ -596,7 +595,6 @@ static void
 write_ready_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *fixture)
 {
 	GOutputStream *stream = G_OUTPUT_STREAM (source);
-	GError *error = NULL;
 	gssize write_size;
 	SoupCacheEntry *entry = fixture->entry;
 
@@ -609,10 +607,8 @@ write_ready_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *
 		return;
 	}
 
-	write_size = g_output_stream_write_finish (stream, result, &error);
-	if (write_size <= 0 || error) {
-		if (error)
-			entry->error = error;
+	write_size = g_output_stream_write_finish (stream, result, &fixture->error);
+	if (write_size <= 0 || fixture->error) {
 		g_output_stream_close_async (stream,
 					     G_PRIORITY_LOW,
 					     entry->cancellable,
@@ -626,14 +622,13 @@ write_ready_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *
 		if (fixture->buffer_queue->length > 0)
 			write_next_buffer (entry, fixture);
 		else {
-			soup_buffer_free (entry->current_writing_buffer);
-			entry->current_writing_buffer = NULL;
+			g_clear_pointer (&fixture->current_writing_buffer, soup_buffer_free);
 
-			if (entry->got_body) {
+			if (fixture->got_body) {
 				/* If we already received 'got-body'
 				   and we have written all the data,
 				   we can close the stream */
-				g_output_stream_close_async (entry->stream,
+				g_output_stream_close_async (fixture->ostream,
 							     G_PRIORITY_LOW,
 							     entry->cancellable,
 							     (GAsyncReadyCallback)close_ready_cb,
@@ -652,13 +647,10 @@ write_next_buffer (SoupCacheEntry *entry, SoupCacheWritingFixture *fixture)
 		return FALSE;
 
 	/* Free the old buffer */
-	if (entry->current_writing_buffer) {
-		soup_buffer_free (entry->current_writing_buffer);
-		entry->current_writing_buffer = NULL;
-	}
-	entry->current_writing_buffer = buffer;
+	g_clear_pointer (&fixture->current_writing_buffer, soup_buffer_free);
+	fixture->current_writing_buffer = buffer;
 
-	g_output_stream_write_async (entry->stream, buffer->data, buffer->length,
+	g_output_stream_write_async (fixture->ostream, buffer->data, buffer->length,
 				     G_PRIORITY_LOW, entry->cancellable,
 				     (GAsyncReadyCallback) write_ready_cb,
 				     fixture);
@@ -684,7 +676,7 @@ msg_got_chunk_cb (SoupMessage *msg, SoupBuffer *chunk, SoupCacheWritingFixture *
 	/* FIXME: remove the error check when we cancel the caching at
 	   the first write error */
 	/* Only write if the entry stream is ready */
-	if (entry->current_writing_buffer == NULL && entry->error == NULL && entry->stream)
+	if (fixture->current_writing_buffer == NULL && fixture->error == NULL && fixture->ostream)
 		write_next_buffer (entry, fixture);
 }
 
@@ -694,9 +686,9 @@ msg_got_body_cb (SoupMessage *msg, SoupCacheWritingFixture *fixture)
 	SoupCacheEntry *entry = fixture->entry;
 	g_return_if_fail (entry);
 
-	entry->got_body = TRUE;
+	fixture->got_body = TRUE;
 
-	if (!entry->stream && fixture->buffer_queue->length > 0)
+	if (!fixture->ostream && fixture->buffer_queue->length > 0)
 		/* The stream is not ready to be written but we still
 		   have data to write, we'll write it when the stream
 		   is opened for writing */
@@ -706,13 +698,13 @@ msg_got_body_cb (SoupMessage *msg, SoupCacheWritingFixture *fixture)
 	if (fixture->buffer_queue->length > 0) {
 		/* If we still have data to write, write it,
 		   write_ready_cb will close the stream */
-		if (entry->current_writing_buffer == NULL && entry->error == NULL && entry->stream)
+		if (fixture->current_writing_buffer == NULL && fixture->error == NULL && fixture->ostream)
 			write_next_buffer (entry, fixture);
 		return;
 	}
 
-	if (entry->stream && entry->current_writing_buffer == NULL)
-		g_output_stream_close_async (entry->stream,
+	if (fixture->ostream && fixture->current_writing_buffer == NULL)
+		g_output_stream_close_async (fixture->ostream,
 					     G_PRIORITY_LOW,
 					     entry->cancellable,
 					     (GAsyncReadyCallback)close_ready_cb,
@@ -720,7 +712,7 @@ msg_got_body_cb (SoupMessage *msg, SoupCacheWritingFixture *fixture)
 }
 
 static gboolean
-soup_cache_entry_remove (SoupCache *cache, SoupCacheEntry *entry)
+soup_cache_entry_remove (SoupCache *cache, SoupCacheEntry *entry, gboolean purge)
 {
 	GList *lru_item;
 
@@ -745,6 +737,14 @@ soup_cache_entry_remove (SoupCache *cache, SoupCacheEntry *entry)
 
 	g_assert (g_list_length (cache->priv->lru_start) == g_hash_table_size (cache->priv->cache));
 
+	/* Free resources */
+	if (purge) {
+		GFile *file = get_file_from_entry (cache, entry);
+		g_file_delete (file, NULL, NULL);
+		g_object_unref (file);
+	}
+	soup_cache_entry_free (entry);
+
 	return TRUE;
 }
 
@@ -803,10 +803,9 @@ make_room_for_new_entry (SoupCache *cache, guint length_to_add)
 		/* Discard entries. Once cancelled resources will be
 		 * freed in close_ready_cb
 		 */
-		if (soup_cache_entry_remove (cache, old_entry)) {
-			soup_cache_entry_free (old_entry, get_file_from_entry (cache, old_entry));
+		if (soup_cache_entry_remove (cache, old_entry, TRUE))
 			lru_entry = cache->priv->lru_start;
-		} else
+		else
 			lru_entry = g_list_next (lru_entry);
 	}
 }
@@ -836,9 +835,7 @@ soup_cache_entry_insert (SoupCache *cache,
 
 	/* Remove any previous entry */
 	if ((old_entry = g_hash_table_lookup (cache->priv->cache, GUINT_TO_POINTER (entry->key))) != NULL) {
-		if (soup_cache_entry_remove (cache, old_entry))
-			soup_cache_entry_free (old_entry, get_file_from_entry (cache, old_entry));
-		else
+		if (!soup_cache_entry_remove (cache, old_entry, TRUE))
 			return FALSE;
 	}
 
@@ -889,27 +886,25 @@ static void
 replace_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *fixture)
 {
 	SoupCacheEntry *entry = fixture->entry;
-	GOutputStream *stream = (GOutputStream *) g_file_replace_finish (G_FILE (source),
-									 result, &entry->error);
+	GOutputStream *ostream = (GOutputStream *) g_file_replace_finish (G_FILE (source),
+									  result, &fixture->error);
 
-	if (g_cancellable_is_cancelled (entry->cancellable) || entry->error) {
-		if (stream)
-			g_object_unref (stream);
+	if (g_cancellable_is_cancelled (entry->cancellable) || fixture->error) {
+		g_clear_object (&ostream);
 		fixture->cache->priv->n_pending--;
 		entry->dirty = FALSE;
-		soup_cache_entry_remove (fixture->cache, entry);
-		soup_cache_entry_free (entry, get_file_from_entry (fixture->cache, entry));
+		soup_cache_entry_remove (fixture->cache, entry, TRUE);
 		soup_cache_writing_fixture_free (fixture);
 		return;
 	}
 
-	entry->stream = stream;
+	fixture->ostream = ostream;
 
 	/* If we already got all the data we have to initiate the
 	 * writing here, since we won't get more 'got-chunk'
 	 * signals
 	 */
-	if (!entry->got_body)
+	if (!fixture->got_body)
 		return;
 
 	/* It could happen that reading the data from server
@@ -918,7 +913,7 @@ replace_cb (GObject *source, GAsyncResult *result, SoupCacheWritingFixture *fixt
 	 */
 	if (!write_next_buffer (entry, fixture))
 		/* Could happen if the resource is empty */
-		g_output_stream_close_async (stream, G_PRIORITY_LOW, entry->cancellable,
+		g_output_stream_close_async (ostream, G_PRIORITY_LOW, entry->cancellable,
 					     (GAsyncReadyCallback) close_ready_cb,
 					     fixture);
 }
@@ -959,17 +954,15 @@ msg_got_headers_cb (SoupMessage *msg, gpointer user_data)
 			return;
 
 		/* Create a new entry, deleting any old one if present */
-		if (entry) {
-			soup_cache_entry_remove (cache, entry);
-			soup_cache_entry_free (entry, get_file_from_entry (cache, entry));
-		}
+		if (entry)
+			soup_cache_entry_remove (cache, entry, TRUE);
 
 		entry = soup_cache_entry_new (cache, msg, request_time, response_time);
 		entry->hits = 1;
 
 		/* Do not continue if it can not be stored */
 		if (!soup_cache_entry_insert (cache, entry, TRUE)) {
-			soup_cache_entry_free (entry, get_file_from_entry (cache, entry));
+			soup_cache_entry_free (entry);
 			return;
 		}
 
@@ -1004,10 +997,8 @@ msg_got_headers_cb (SoupMessage *msg, gpointer user_data)
 	} else if (cacheable & SOUP_CACHE_INVALIDATES) {
 		entry = soup_cache_entry_lookup (cache, msg);
 
-		if (entry) {
-			if (soup_cache_entry_remove (cache, entry))
-				soup_cache_entry_free (entry, get_file_from_entry (cache, entry));
-		}
+		if (entry)
+			soup_cache_entry_remove (cache, entry, TRUE);
 	} else if (cacheable & SOUP_CACHE_VALIDATES) {
 		entry = soup_cache_entry_lookup (cache, msg);
 
@@ -1127,11 +1118,7 @@ static void
 remove_cache_item (gpointer data,
 		   gpointer user_data)
 {
-	SoupCache *cache = (SoupCache *) user_data;
-	SoupCacheEntry *entry = (SoupCacheEntry *) data;
-
-	if (soup_cache_entry_remove (cache, entry))
-		soup_cache_entry_free (entry, NULL);
+	soup_cache_entry_remove ((SoupCache *) user_data, (SoupCacheEntry *) data, FALSE);
 }
 
 static void
@@ -1502,19 +1489,38 @@ static void
 clear_cache_item (gpointer data,
 		  gpointer user_data)
 {
-	SoupCache *cache = (SoupCache *) user_data;
-	SoupCacheEntry *entry = (SoupCacheEntry *) data;
+	soup_cache_entry_remove ((SoupCache *) user_data, (SoupCacheEntry *) data, TRUE);
+}
 
-	if (soup_cache_entry_remove (cache, entry))
-		soup_cache_entry_free (entry, get_file_from_entry (cache, entry));
+static void
+clear_cache_files (SoupCache *cache)
+{
+	GFileInfo *file_info;
+	GFileEnumerator *file_enumerator;
+	GFile *cache_dir_file = g_file_new_for_path (cache->priv->cache_dir);
+
+	file_enumerator = g_file_enumerate_children (cache_dir_file, G_FILE_ATTRIBUTE_STANDARD_NAME,
+						     G_FILE_QUERY_INFO_NONE, NULL, NULL);
+	if (file_enumerator) {
+		while ((file_info = g_file_enumerator_next_file (file_enumerator, NULL, NULL)) != NULL) {
+			const char *filename = g_file_info_get_name (file_info);
+
+			if (strcmp (filename, SOUP_CACHE_FILE) != 0) {
+				GFile *cache_file = g_file_get_child (cache_dir_file, filename);
+				g_file_delete (cache_file, NULL, NULL);
+				g_object_unref (cache_file);
+			}
+		}
+		g_object_unref (file_enumerator);
+	}
+	g_object_unref (cache_dir_file);
 }
 
 /**
  * soup_cache_clear:
  * @cache: a #SoupCache
  *
- * Will remove all entries in the @cache plus all the cache files
- * associated with them.
+ * Will remove all entries in the @cache plus all the cache files.
  *
  * Since: 2.34
  */
@@ -1530,6 +1536,9 @@ soup_cache_clear (SoupCache *cache)
 	entries = g_hash_table_get_values (cache->priv->cache);
 	g_list_foreach (entries, clear_cache_item, cache);
 	g_list_free (entries);
+
+	/* Remove also any file not associated with a cache entry. */
+	clear_cache_files (cache);
 }
 
 SoupMessage *
@@ -1575,18 +1584,6 @@ soup_cache_generate_conditional_request (SoupCache *cache, SoupMessage *original
 	return msg;
 }
 
-#define OLD_SOUP_CACHE_FILE "soup.cache"
-#define SOUP_CACHE_FILE "soup.cache2"
-
-#define SOUP_CACHE_HEADERS_FORMAT "{ss}"
-#define SOUP_CACHE_PHEADERS_FORMAT "(sbuuuuuqa" SOUP_CACHE_HEADERS_FORMAT ")"
-#define SOUP_CACHE_ENTRIES_FORMAT "(qa" SOUP_CACHE_PHEADERS_FORMAT ")"
-
-/* Basically the same format than above except that some strings are
-   prepended with &. This way the GVariant returns a pointer to the
-   data instead of duplicating the string */
-#define SOUP_CACHE_DECODE_HEADERS_FORMAT "{&s&s}"
-
 static void
 pack_entry (gpointer data,
 	    gpointer user_data)
@@ -1597,7 +1594,7 @@ pack_entry (gpointer data,
 	GVariantBuilder *entries_builder = (GVariantBuilder *)user_data;
 
 	/* Do not store non-consolidated entries */
-	if (entry->dirty || entry->current_writing_buffer != NULL || !entry->key)
+	if (entry->dirty || !entry->key)
 		return;
 
 	g_variant_builder_open (entries_builder, G_VARIANT_TYPE (SOUP_CACHE_PHEADERS_FORMAT));
@@ -1650,30 +1647,6 @@ soup_cache_dump (SoupCache *cache)
 	g_variant_unref (cache_variant);
 }
 
-static void
-clear_cache_files (SoupCache *cache)
-{
-	GFileInfo *file_info;
-	GFileEnumerator *file_enumerator;
-	GFile *cache_dir_file = g_file_new_for_path (cache->priv->cache_dir);
-
-	file_enumerator = g_file_enumerate_children (cache_dir_file, G_FILE_ATTRIBUTE_STANDARD_NAME,
-						     G_FILE_QUERY_INFO_NONE, NULL, NULL);
-	if (file_enumerator) {
-		while ((file_info = g_file_enumerator_next_file (file_enumerator, NULL, NULL)) != NULL) {
-			const char *filename = g_file_info_get_name (file_info);
-
-			if (strcmp (filename, SOUP_CACHE_FILE) != 0) {
-				GFile *cache_file = g_file_get_child (cache_dir_file, filename);
-				g_file_delete (cache_file, NULL, NULL);
-				g_object_unref (cache_file);
-			}
-		}
-		g_object_unref (file_enumerator);
-	}
-	g_object_unref (cache_dir_file);
-}
-
 void
 soup_cache_load (SoupCache *cache)
 {
@@ -1741,7 +1714,7 @@ soup_cache_load (SoupCache *cache)
 		entry->status_code = status_code;
 
 		if (!soup_cache_entry_insert (cache, entry, FALSE))
-			soup_cache_entry_free (entry, get_file_from_entry (cache, entry));
+			soup_cache_entry_free (entry);
 	}
 
 	cache->priv->lru_start = g_list_reverse (cache->priv->lru_start);
