@@ -16,6 +16,7 @@
 #include "soup-session-private.h"
 #include "soup-message-private.h"
 #include "soup-message-queue.h"
+#include "soup-misc-private.h"
 
 /**
  * SECTION:soup-session-async
@@ -37,6 +38,7 @@ G_DEFINE_TYPE (SoupSessionAsync, soup_session_async, SOUP_TYPE_SESSION)
 
 typedef struct {
 	SoupSessionAsync *sa;
+	GSList *sources;
 	gboolean disposed;
 
 } SoupSessionAsyncPrivate;
@@ -54,8 +56,14 @@ static void
 soup_session_async_dispose (GObject *object)
 {
 	SoupSessionAsyncPrivate *priv = SOUP_SESSION_ASYNC_GET_PRIVATE (object);
+	GSList *iter;
 
 	priv->disposed = TRUE;
+	for (iter = priv->sources; iter; iter = iter->next) {
+		g_source_destroy (iter->data);
+		g_source_unref (iter->data);
+	}
+	g_clear_pointer (&priv->sources, g_slist_free);
 
 	G_OBJECT_CLASS (soup_session_async_parent_class)->dispose (object);
 }
@@ -132,7 +140,6 @@ tunnel_complete (SoupMessageQueueItem *item)
 	soup_message_queue_item_unref (item->related);
 	soup_session_unqueue_item (session, item);
 	soup_message_queue_item_unref (item);
-	g_object_unref (session);
 }
 
 static void
@@ -202,7 +209,6 @@ got_connection (SoupConnection *conn, guint status, gpointer user_data)
 		soup_connection_disconnect (conn);
 		do_idle_run_queue (session);
 		soup_message_queue_item_unref (item);
-		g_object_unref (session);
 		return;
 	}
 
@@ -221,7 +227,6 @@ got_connection (SoupConnection *conn, guint status, gpointer user_data)
 
 		do_idle_run_queue (session);
 		soup_message_queue_item_unref (item);
-		g_object_unref (session);
 		return;
 	}
 
@@ -241,7 +246,6 @@ got_connection (SoupConnection *conn, guint status, gpointer user_data)
 			  G_CALLBACK (connection_closed), session);
 	run_queue ((SoupSessionAsync *)session);
 	soup_message_queue_item_unref (item);
-	g_object_unref (session);
 }
 
 static void
@@ -274,7 +278,6 @@ process_queue_item (SoupMessageQueueItem *item,
 
 			item->state = SOUP_MESSAGE_CONNECTING;
 			soup_message_queue_item_ref (item);
-			g_object_ref (session);
 			soup_connection_connect_async (item->conn, item->cancellable,
 						       got_connection, item);
 			return;
@@ -299,14 +302,14 @@ process_queue_item (SoupMessageQueueItem *item,
 			if (item->state != SOUP_MESSAGE_FINISHED)
 				break;
 
-			g_object_ref (session);
+			soup_message_queue_item_ref (item);
 			soup_session_unqueue_item (session, item);
 			if (item->callback)
 				item->callback (session, item->msg, item->callback_data);
 			else if (item->new_api)
 				send_request_finished (session, item);
 			do_idle_run_queue (session);
-			g_object_unref (session);
+			soup_message_queue_item_unref (item);
 			return;
 
 		default:
@@ -359,12 +362,17 @@ static gboolean
 idle_run_queue (gpointer user_data)
 {
 	SoupSessionAsyncPrivate *priv = user_data;
+	GSource *source;
 
 	if (priv->disposed)
 		return FALSE;
 
+	source = g_main_current_source ();
+	priv->sources = g_slist_remove (priv->sources, source);
+
 	/* Ensure that the source is destroyed before running the queue */
-	g_source_destroy (g_main_current_source ());
+	g_source_destroy (source);
+	g_source_unref (source);
 
 	run_queue (priv->sa);
 	return FALSE;
@@ -389,7 +397,8 @@ do_idle_run_queue (SoupSession *session)
 	if (source)
 		return;
 
-	source = soup_add_completion (async_context, idle_run_queue, priv);
+	source = soup_add_completion_reffed (async_context, idle_run_queue, priv);
+	priv->sources = g_slist_prepend (priv->sources, source);
 }
 
 static void
@@ -543,7 +552,7 @@ send_request_finished (SoupSession *session, SoupMessageQueueItem *item)
 		return;
 	}
 
-	mostream = g_object_get_data (G_OBJECT (item->msg), "SoupSessionAsync:ostream");
+	mostream = g_object_get_data (G_OBJECT (item->result), "SoupSessionAsync:ostream");
 	if (mostream) {
 		gpointer data;
 		gssize size;
@@ -576,9 +585,14 @@ send_async_spliced (GObject *source, GAsyncResult *result, gpointer user_data)
 {
 	SoupMessageQueueItem *item = user_data;
 	GInputStream *istream = g_object_get_data (source, "istream");
-
 	GError *error = NULL;
 
+	/* It should be safe to call the sync close() method here since
+	 * the message body has already been written.
+	 */
+	g_input_stream_close (istream, NULL, NULL);
+	g_object_unref (istream);
+
 	/* If the message was cancelled, it will be completed via other means */
 	if (g_cancellable_is_cancelled (item->cancellable) ||
 	    !item->result) {
@@ -589,14 +603,11 @@ send_async_spliced (GObject *source, GAsyncResult *result, gpointer user_data)
 	if (g_output_stream_splice_finish (G_OUTPUT_STREAM (source),
 					   result, &error) == -1) {
 		send_request_return_result (item, NULL, error);
+		soup_message_queue_item_unref (item);
 		return;
 	}
 
-	/* Otherwise either restarted or finished will eventually be called.
-	 * It should be safe to call the sync close() method here since
-	 * the message body has already been written.
-	 */
-	g_input_stream_close (istream, NULL, NULL);
+	/* Otherwise either restarted or finished will eventually be called. */
 	do_idle_run_queue (item->session);
 	soup_message_queue_item_unref (item);
 }
@@ -612,11 +623,10 @@ send_async_maybe_complete (SoupMessageQueueItem *item,
 
 		/* Message may be requeued, so gather the current message body... */
 		ostream = g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
-		g_object_set_data_full (G_OBJECT (item->msg), "SoupSessionAsync:ostream",
+		g_object_set_data_full (G_OBJECT (item->result), "SoupSessionAsync:ostream",
 					ostream, g_object_unref);
 
-		g_object_set_data_full (G_OBJECT (ostream), "istream",
-					stream, g_object_unref);
+		g_object_set_data (G_OBJECT (ostream), "istream", stream);
 
 		/* Give the splice op its own ref on item */
 		soup_message_queue_item_ref (item);
